#ifdef TRY_TELNET
/*****************************************************************************
 *                                  _   _ ____  _     
 *  Project                     ___| | | |  _ \| |    
 *                             / __| | | | |_) | |    
 *                            | (__| |_| |  _ <| |___ 
 *                             \___|\___/|_| \_\_____|
 *
 *  The contents of this file are subject to the Mozilla Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/MPL/
 *
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *
 *  The Original Code is Curl.
 *
 *  The Initial Developer of the Original Code is Daniel Stenberg.
 *
 *  Portions created by the Initial Developer are Copyright (C) 1998.
 *  All Rights Reserved.
 *
 * ------------------------------------------------------------
 * Main author:
 * - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
 *
 * 	http://www.fts.frontec.se/~dast/curl/
 *
 * $Source: /cvs/curl/curl/lib/telnet.c,v $
 * $Revision: 1.2 $
 * $Date: 1999/05/16 17:20:03 $
 * $Author: dast $
 * $State: Exp $
 * $Locker:  $
 *
 * ------------------------------------------------------------
 ****************************************************************************/

/* -- WIN32 approved -- */
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <errno.h>

#include "../include/setup.h"

#if defined(WIN32) && !defined(__GNUC__)
#include <winsock.h>
#include <time.h>
#include <io.h>
#else
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/resource.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <netdb.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <signal.h>

#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif

#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif


#endif

#include "urldata.h"
#include "../include/curl.h"
#include "download.h"
#include "sendf.h"
#include "formdata.h"
#include "progress.h"

#define _MPRINTF_REPLACE /* use our functions only */
#include "../include/mprintf.h"

  /***********************************************************************
   *
   * Telnet.
   *
   * - Require user+password.
   *
   * 1. Negotiation.
   * 2. Read until login: prompt.
   * 3. Send login.
   * 4. Read until password: prompt.
   * 5. Send password.
   * 6. Send extra command (echo ---)
   * 7. Wait for extra command output, when recieived we know we're in.
   * 8. Run command; echo <boundary>
   * 9. Eat everything until the first newline
   * 10. Dump everything
   * 11. If <boundary> is found, abort.
   *
   * Still very experimental.
   *
   **********************************************************************/

/* for telnet testing only */

char *telnetin(struct UrlData *data,
	       unsigned char inbyte)
{
#define TELNET_SIZE 256
  static char inbuffer[TELNET_SIZE+1];
  static int index=0;

  if(10 == inbyte) {
    /* newline, we reset buffer */
    index=0;
    inbuffer[0]=0;
#if 0
    fprintf(stderr, "[NOLL]\n");
#endif
  }
  else if(' ' > inbyte) {
    /* ignore low ASCII numbers */
#if 0
    fprintf(stderr, "[IGNORE] %u\n", inbyte);
#endif
  }
  else if(index < TELNET_SIZE) {
#if 0
    fprintf(stderr, "In:%c\n", inbyte);
#endif
    inbuffer[index]=inbyte;
    inbuffer[index+1]=0; /* zero terminate all times to allow string
			    functions to operate on buffer */
    index++;
  }
  /* else
     buffer is full, skip it */
  return inbuffer;
}
  
UrgError telnet(struct UrlData *data, char *ppath)
{
  typedef enum {
    TELNET_CONNECT,
    TELNET_LOGIN,
    TELNET_PASSWD,
    TELNET_INSIDE,
    TELNET_DUMP
  } TelnetState;

#define TELCMDS
#define TELOPTS
#include <arpa/telnet.h>
  unsigned char telopt[] =
  {IAC,WONT,TELOPT_TTYPE,
   IAC,DO, TELOPT_SGA,
   IAC,WONT,TELOPT_XDISPLOC,
   IAC,WONT,TELOPT_NAWS,
   IAC,WONT,TELOPT_OLD_ENVIRON,
   IAC,WONT,TELOPT_NEW_ENVIRON,

   IAC,DONT,TELOPT_ECHO};
    
  TelnetState state=TELNET_CONNECT;

  char *endstring;
  int newlines=0;
  int endlen;
  int endmatch=0;

  swrite(data->firstsocket,telopt,
         sizeof(telopt));   /* kludge some telnet negotiation */

  while (1) {
    char *buffer;
    char *ptr;

    sread(data->firstsocket, data->buffer, 1);

    if((unsigned char)data->buffer[0] == IAC) {
      /* 251 - WILL
         252 - WONT
         253 - DO
         254 - DONT
         255 - IAC */
      sread(data->firstsocket, data->buffer, 2);
#if 0
      fprintf(stderr, "IAC %u %u\n",
              (unsigned char)data->buffer[0],
              (unsigned char)data->buffer[1]);
#else
      fprintf(stderr, "IAC %s %s\n",
              telcmds[(unsigned char)data->buffer[0]-TELCMD_FIRST],
              telopts[(unsigned char)data->buffer[1]-TELOPT_FIRST]);
#endif
    }
    else {
      buffer = telnetin(data, data->buffer[0]);

      if(state == TELNET_DUMP) {
        if(newlines) {
          if(!*buffer) {
            /* newline indication, decrease */
            newlines--;
          }
        }
        else {

	    
          if(data->buffer[0] == endstring[endmatch]) {
            endmatch++; /* one (more) letter matches */

            if(endmatch == endlen) {
              /* this is end of data */
              free(endstring);
              break;
            }
          }
          else {
            if(endmatch) {
              data->fwrite(endstring, endmatch, 1, data->out);
              endmatch=0;
            }
            data->fwrite(data->buffer, 1, 1, data->out);
            /* printf("%c", data->buffer[0]); */
          }
        }
      }

      /*	fprintf(stderr, "buffer: %s\n", buffer); */

      if((state == TELNET_CONNECT) &&
         strequal("login: ", buffer)) {
        infof(data, "Got the login: prompt!\n");
        sendf(data->firstsocket, data, "%s\r\n", data->user); 
        state=TELNET_LOGIN; /* we have passed login */
      }
      else if((state == TELNET_LOGIN) &&
              strequal("Password: ", buffer)) {
        infof(data, "Got the Password: prompt!\n");
        sendf(data->firstsocket, data, "%s\r\n", data->passwd); 
        state=TELNET_PASSWD; /* we have passed password */

        sendf(data->firstsocket, data, "echo ---\r\n"); 
      }
      else if((state == TELNET_PASSWD) &&
              strequal("---", buffer)) {
        infof(data, "We're logged in!\n");
        endstring = MakeFormBoundary();

        sendf(data->firstsocket, data, "%s;echo %s\r\n",
              ppath+1, endstring);
        state=TELNET_INSIDE;
      }
      else if((state == TELNET_INSIDE) &&
              (ptr = strstr(buffer, ppath+1)) ) {
        endlen = strlen(endstring);
        endmatch=0;

        state = TELNET_DUMP; newlines=1; /* swallow 2 newlines, then go */
      }
    }
  }
  return URG_OK;
}

#endif
